//+------------------------------------------------------------------+
//|                                                   TickZigZag.mq5 |
//|                        Copyright 2017, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"

#define SIGN(i) ((i<0) ? -1 : 1)
//+------------------------------------------------------------------+
//| sElem structure                                                  |
//+------------------------------------------------------------------+
struct sElem
  {
   datetime          time;
   long              index;
   double            value;
   int               sign;
  };
//+------------------------------------------------------------------+
//| CWaveQueue class                                                 |
//+------------------------------------------------------------------+
class CWaveQueue
  {
private:
   int               m_index;
   int               m_count,m_size;
   int               m_head,m_tail;
   //---
   int               m_sign_changed;
   int               m_wave_sign;
   //---
   sElem             m_data[];
   //---
   void              ReadElement(sElem &value);
   void              WriteElement(sElem &value);
   int               AdvWriteElement(sElem &elem);
public:
   //--- class constructor
                     CWaveQueue();
                    ~CWaveQueue() {ArrayResize(m_data,0);             }
   //--- create and add values
   void              Create(const int size);
   bool              AddValue(const datetime time,const double value);
   //--- check
   bool              IsFull()                 {return((m_tail+1)%m_size==m_head); }
   bool              IsEmpty()                {return(m_head==m_tail);            }
   //--- access to the elements
   int               Count() {return(m_count);                   }
   void              GetElement(int index,sElem &value);
   //--- additional methods
   int               WaveSign()               {return(m_wave_sign);               }
   bool              IsSignChanged()          {return(m_sign_changed);            }
  };
//+------------------------------------------------------------------+
//| CWaveQueue class constructor                                     |
//+------------------------------------------------------------------+
CWaveQueue::CWaveQueue()
  {
   m_head=0;
   m_tail=0;
   m_count=0;
   m_size=0;
   m_index=0;
//---
   m_wave_sign=0;
  };
//+------------------------------------------------------------------+
//| Create                                                           |
//+------------------------------------------------------------------+
CWaveQueue::Create(const int size)
  {
   m_size=size+1;
   ArrayResize(m_data,m_size);
  }
//+------------------------------------------------------------------+
//| WriteElement                                                     |
//+------------------------------------------------------------------+
void CWaveQueue::WriteElement(sElem &value)
  {
   m_data[m_tail]=value;
   m_tail=(m_tail+1)%m_size;
   if(m_tail==m_head)
      m_head=(m_head+1)%m_size;
   m_count++;
   if(m_count>m_size-1)
      m_count=m_size-1;
   m_index++;
  }
//+------------------------------------------------------------------+
//| ReadElement                                                      |
//+------------------------------------------------------------------+
void CWaveQueue::ReadElement(sElem &value)
  {
   value=m_data[m_head];
   m_head=(m_head+1)%m_size;
   m_count--;
   if(m_count<0)
      m_count=0;
  }
//+------------------------------------------------------------------+
//| GetElement                                                       |
//+------------------------------------------------------------------+
void CWaveQueue::GetElement(const int index,sElem &value)
  {
   int real_index=(m_head+index)%m_size;
   value=m_data[real_index];
  }
//+------------------------------------------------------------------+
//| AddValue                                                         |
//+------------------------------------------------------------------+
bool CWaveQueue::AddValue(const datetime time,const double value)
  {
   sElem elem;
   elem.time=time;
   elem.value=value;
   elem.index=m_index;
   elem.sign=m_wave_sign;
   m_sign_changed=(AdvWriteElement(elem)==2);
//---
   return(m_sign_changed);
  }
//+------------------------------------------------------------------+
//| AdvWriteElement                                                  |
//| (0-added,1 - modified,2 - direction changed                      |
//+------------------------------------------------------------------+
int CWaveQueue::AdvWriteElement(sElem &elem)
  {
//--- just add new value
   if(IsEmpty() || (Count()==1))
     {
      WriteElement(elem);
      return(0);
     }
//--- two previous elements
   sElem prev1,prev2;
//-- get two previous elements
   GetElement(Count()-1,prev1);
   GetElement(Count()-2,prev2);
//---
   double delta1=elem.value-prev1.value;
   double delta2=prev1.value-prev2.value;
//--- compare with previous value, return if equal
   if(delta1==0)
     {
      //--- modify the last element
      m_data[(m_head+Count()-1)%m_size]=elem;
      return(1);
     }
//--- calc wave direction
   if(delta1>0)
      m_wave_sign=1;
   else
   if(delta1<0)
      m_wave_sign=-1;
   elem.sign=m_wave_sign;
//--- check change of the direction
   if(SIGN(delta1)!=SIGN(delta2))
     {
      //--- add new element
      WriteElement(elem);
      return(2);
     }
   else
     {
      //--- modify the last element
      m_data[(m_head+Count()-1)%m_size]=elem;
      return(1);
     }
   return(0);
  }

string ExtSymbol="EURUSD";
double ExtSpread;
string ExtCustom_symbol="EURUSD.ZigZag2";
CWaveQueue ExtWaveQueue;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   ExtWaveQueue.Create(100);
//---
   string custom_symbol=ExtCustom_symbol;
   string custom_symbol_path="Custom\\ZigZag\\"+custom_symbol;
//---
   bool is_custom=false;
   bool res=SymbolSelect(custom_symbol,true);
   if(res)
      is_custom=(bool)SymbolInfoInteger(custom_symbol,SYMBOL_CUSTOM);
//--- create custom symbol
   if(!res)
     {
      if(!CustomSymbolCreate(custom_symbol,custom_symbol_path))
        {
         Print("cannot create custom symbol ",custom_symbol);
         return(INIT_FAILED);
        }
      //---
      CustomSymbolSetDouble(custom_symbol,SYMBOL_POINT,_Point);
      CustomSymbolSetInteger(custom_symbol,SYMBOL_DIGITS,_Digits);
      CustomSymbolSetInteger(custom_symbol,SYMBOL_SPREAD,0);
      is_custom=true;
      //---
      if(!SymbolSelect(custom_symbol,true))
        {
         Print("cannot select custom symbol ",custom_symbol);
         return(INIT_FAILED);
        }
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   MqlTick tick;
   if(SymbolInfoTick(ExtSymbol,tick))
     {
      //double value=0.5*(tick.bid+tick.ask);
      double value=tick.bid;
      datetime time=tick.time;
      //---
      if(ExtWaveQueue.AddValue(time,value))
        {
         Print("Adding new tick :",tick.bid,"/",tick.ask);
         MqlTick new_tick[1]={0};
         new_tick[0].bid=tick.bid;
         new_tick[0].ask=tick.bid+_Point;
         new_tick[0].time=time;
         //new_tick[0].bid=tick.bid;
         //new_tick[0].ask=tick.ask;
         int cnt=CustomTicksAdd(ExtCustom_symbol,new_tick);
        }
     }
  }
//+------------------------------------------------------------------+
